@@ -105,9 +105,16 @@ makeCovEst1 <- function(u_h,ZAlist,cum_n_u_h,prev_LMatrices,
       parscale <- (upperb-lowerb)
       if (TRUE) {
         objfn_nloptr <- function(x) { return( - objfn(x)) }
-        nloptr_controls <- list(algorithm="NLOPT_LN_BOBYQA",xtol_rel=1.0e-4,maxeval=-1,print_level=0) ## DEFAULT
-        optr <- nloptr(x0=init,eval_f=objfn_nloptr,lb=lowerb,ub=upperb,
+        nloptr_controls <- list(algorithm="NLOPT_LN_BOBYQA",xtol_rel=1.0e-4,maxeval=100,print_level=0) ## DEFAULT
+        optr <- nloptr::nloptr(x0=init,eval_f=objfn_nloptr,lb=lowerb,ub=upperb,
                        opts=nloptr_controls)
+        while (optr$status==5) { ## 5 => termination bc maxeval has been reached
+          prevlik <- optr$objective
+          reinit <- pmax(lowerb,pmin(upperb,optr$solution))
+          optr <- nloptr::nloptr(x0=reinit,eval_f=objfn_nloptr,lb=lowerb,ub=upperb,
+                                 opts=nloptr_controls)
+          if (optr$objective > prevlik-optr$options$ftol_abs ) break ## no progress in <= maxeval iterations
+        }
         optr$par <- optr$solution
       } else {
         ################# OPTIM
